package com.jianshui.invoice.mapper;

import java.util.List;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.jianshui.common.core.domain.entity.Companyservice;
import com.jianshui.invoice.domain.Invoice;
import com.jianshui.invoice.domain.InvoiceDetail;
import org.apache.ibatis.annotations.Param;

/**
 * 已开发票列表Mapper接口
 *
 * @author Gong Quanlin
 * @date 2022-03-08
 */
public interface InvoiceMapper extends BaseMapper<Invoice> {
    /**
     * 查询已开发票列表
     *
     * @param id 已开发票列表主键
     * @return 已开发票列表
     */
    public Invoice selectInvoiceById(Long id);

    /**
     * 查询已开发票列表列表
     *
     * @param invoice 已开发票列表
     * @return 已开发票列表集合
     */
    public List<Invoice> selectInvoiceList(Invoice invoice);

    /**
     * 新增已开发票列表
     *
     * @param invoice 已开发票列表
     * @return 结果
     */
    public int insertInvoice(Invoice invoice);

    /**
     * 修改已开发票列表
     *
     * @param invoice 已开发票列表
     * @return 结果
     */
    public int updateInvoice(Invoice invoice);

    /**
     * 删除已开发票列表
     *
     * @param id 已开发票列表主键
     * @return 结果
     */
    public int deleteInvoiceById(Long id);

    /**
     * 批量删除已开发票列表
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteInvoiceByIds(Long[] ids);

    /**
     * 批量删除已开发票明细
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteInvoiceDetailByInvoiceIds(Long[] ids);

    /**
     * 批量新增已开发票明细
     *
     * @param invoiceDetailList 已开发票明细列表
     * @return 结果
     */
    public int batchInvoiceDetail(List<InvoiceDetail> invoiceDetailList);


    /**
     * 通过已开发票列表主键删除已开发票明细信息
     *
     * @param id 已开发票列表ID
     * @return 结果
     */
    public int deleteInvoiceDetailByInvoiceId(Long id);


    /**
     * 根据系统单号查询发票
     *
     * @param companyId
     * @param systemOrderNo
     * @return
     */
    public Invoice selectBySystemOrderNo(@Param(("companyId")) Long companyId, @Param("system_orderno") String systemOrderNo);


    /**
     * 查询发票信息，目前用于控制台发票查询返回，整合selectInvoiceById、selectBySystemOrderNo、selectByFpdmFphm，有新加字段的时候一起改着这个SQL
     * @return
     */
    public Invoice selectInvoiceAllIn(Invoice invoice);

    /**
     * 根据发票代码和号码进行查询
     *
     * @param companyId
     * @param fpdm
     * @param fphm
     * @return
     */
    public Invoice selectByFpdmFphm(@Param(("companyId")) Long companyId, @Param("fpdm") String fpdm, @Param("fphm") String fphm);

    public Invoice selectInvoiceByBillInfoId(Long id);
}
